@@ -2,7 +2,7 @@
  *
  * Information about ssh channels.
  *
- * $Id: channel.h,v 1.70 2001/09/26 16:48:44 nisse Exp $
+ * $Id: channel.h,v 1.71 2002/03/14 13:11:40 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -204,9 +204,9 @@ struct channel_request_info
        ; method as is traditionally used for allocation of unix file 
        ; descriptors.
 
-       ; Channel numbers can be reserved before there is any actual channel
-       ; assigned to them. So the channels table is not enough for keeping track of which
-       ; numbers are in use.
+       ; Channel numbers can be reserved before there is any actual
+       ; channel assigned to them. So the channels table is not enough
+       ; for keeping track of which numbers are in use.
        (in_use space UINT8)
 
        ; Allocated size of the arrays.
