@@ -2,7 +2,7 @@
  *
  * General channel type for forwarding data to an fd
  *
- * $Id: channel_forward.h,v 1.1 2001/08/28 18:33:07 nisse Exp $ */
+ * $Id: channel_forward.h,v 1.2 2001/08/30 21:02:06 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -40,11 +40,15 @@
        (socket object lsh_fd)))
 */
 
-struct ssh_channel *
+void
+init_channel_forward(struct channel_forward *self,
+		     struct lsh_fd *socket, UINT32 initial_window);
+
+struct channel_forward *
 make_channel_forward(struct lsh_fd *socket, UINT32 initial_window);
 
 void
-channel_forward_start_io(struct ssh_channel *c);
+channel_forward_start_io(struct channel_forward *channel_forward);
 
 
 #endif /* LSH_CHANNEL_FORWARD_H_INCLUDED */
