@@ -2,7 +2,7 @@
  *
  * Translate local characterset to and from utf8.
  *
- * $Id: charset.c,v 1.6 1999/02/25 00:33:11 nisse Exp $
+ * $Id: charset.c,v 1.7 2001/04/05 16:16:32 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -66,7 +66,8 @@ int ucs4_to_local(UINT32 c)
     };
 }
 
-struct lsh_string *local_to_utf8(struct lsh_string *s, int free)
+struct lsh_string *
+local_to_utf8(struct lsh_string *s, int free)
 {
   switch (local_charset)
     {
