@@ -1,6 +1,6 @@
 /* client.c
  *
- * $Id: client.c,v 1.137 2002/02/27 14:32:51 nisse Exp $ */
+ * $Id: client.c,v 1.140 2002/04/03 10:56:00 _cvs_pont Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -411,6 +411,8 @@ make_client_session(struct client_options *options);
 
 #define OPT_FORK_STDIO 0x213
 
+#define OPT_SUBSYSTEM 0x214
+
 void
 init_client_options(struct client_options *self,
 		    struct randomness *random,
@@ -499,8 +501,13 @@ client_options[] =
   { "background", 'B', NULL, 0, "Put process into the background. Implies -N.", 0 },
   { "execute", 'E', "command", 0, "Execute a command on the remote machine", 0 },
   { "shell", 'S', "command", 0, "Spawn a remote shell", 0 },
-  { "subsystem", 'C', "subsystem-name", 0, "Connect to given subsystem", 0 },
-
+  { "subsystem", OPT_SUBSYSTEM, "subsystem-name", 0,
+#if WITH_PTY_SUPPORT 
+    "Connect to given subsystem. Implies --no-pty.",
+#else
+    "Connect to given subsystem.",
+#endif
+    0 },
   /* { "gateway", 'G', NULL, 0, "Setup a local gateway", 0 }, */
   { NULL, 0, NULL, 0, "Universal not:", 0 },
   { "no", 'n', NULL, 0, "Inverts the effect of the next modifier", 0 },
@@ -1157,10 +1164,15 @@ client_argp_parser(int key, char *arg, struct argp_state *state)
       client_add_action(options, client_shell_session(options));
       break;
 
-    case 'C':
+    case OPT_SUBSYSTEM:
       client_add_action(options,
 			client_subsystem_session(options,
 						 ssh_format("%lz", arg)));
+
+      options->start_shell = 0;
+#if WITH_PTY_SUPPORT
+      options->with_pty = 0;
+#endif
       break;
 
     case 'L':
