@@ -1,6 +1,6 @@
 /* client_session.c
  *
- * $Id: client_session.c,v 1.17 2002/03/20 13:38:55 nisse Exp $
+ * $Id: client_session.c,v 1.18 2002/05/02 20:52:32 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -217,7 +217,7 @@ make_client_session_channel(struct lsh_fd *in,
   self->super.rec_window_size = initial_window;
 
   /* FIXME: Make maximum packet size configurable */
-  self->super.rec_max_packet = SSH_MAX_PACKET - SSH_CHANNEL_MAX_PACKET_FUZZ;
+  self->super.rec_max_packet = SSH_MAX_PACKET;
 
   self->super.request_types = make_alist(0, -1);
 
