@@ -1,6 +1,6 @@
 /* exception.c
  *
- * $Id: exception.c,v 1.14 2000/05/28 00:06:22 nisse Exp $ */
+ * $Id: exception.c,v 1.16 2001/03/14 18:24:37 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -46,9 +46,6 @@ do_default_handler(struct exception_handler *ignored UNUSED,
 struct exception_handler default_exception_handler =
 STATIC_EXCEPTION_HANDLER(do_default_handler, NULL);
 
-struct exception dummy_exception =
-STATIC_EXCEPTION(EXC_DUMMY, "dummy");
-
 static void
 do_ignore_exception_handler(struct exception_handler *self UNUSED,
 			    const struct exception *e UNUSED)
@@ -120,15 +117,6 @@ make_report_exception_handler(struct report_exception_info *info,
   return &self->super;
 }
 
-#if 0
-/* Command to report and ignore an exception given as argument. */
-COMMAND_SIMPLE(report_exception_command)
-{
-  CAST_SUBTYPE(exception, x, a);
-  werror("%z\n", x->msg);
-}
-#endif
-
 struct exception *make_simple_exception(UINT32 type, const char *msg)
 {
   NEW(exception, e);
