@@ -2,7 +2,7 @@
  *
  * Create a packet from a format string and arguments.
  *
- * $Id: format.h,v 1.35 2002/01/14 01:09:27 nisse Exp $ */
+ * $Id: format.h,v 1.37 2002/04/03 21:57:21 _cvs_pont Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -106,4 +106,10 @@ int lsh_string_eq_l(const struct lsh_string *a, UINT32 length, const UINT8 *b);
 int lsh_string_prefixp(const struct lsh_string *prefix,
 		       const struct lsh_string *s);
 
+struct lsh_string*
+lsh_string_colonize(struct lsh_string *s, int every, int freeflag);
+
+struct lsh_string*
+lsh_string_bubblebabble(struct lsh_string *s, int freeflag);
+
 #endif /* LSH_FORMAT_H_INCLUDED */
