@@ -1,6 +1,6 @@
 /* gateway.c
  *
- * $Id: gateway.c,v 1.4 2000/11/26 23:25:51 nisse Exp $
+ * $Id: gateway.c,v 1.6 2001/04/08 15:15:13 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -27,8 +27,6 @@
 #include "format.h"
 #include "io.h"
 
-#include "gateway.c.x"
-
 /* A gateway is a mechanism to delegate some channels to a separate
  * process. The main lsh process opens a unix domain socket, and other
  * processes can connect and read and write cleartext ssh packets.
@@ -87,15 +85,15 @@ make_gateway_address(char *local_user, char *remote_user,
   if (check_string(local_user)
       && check_string(remote_user)
       && check_string_l(target->ip->length, target->ip->data))
-    return make_local_info(ssh_cformat("%lz/x-lsh-%lz", tmp, local_user),
-			   ssh_cformat("%lS:%lz", target->ip, remote_user));
+    return make_local_info(ssh_format("%lz/x-lsh-%lz", tmp, local_user),
+			   ssh_format("%lS:%lz", target->ip, remote_user));
   else
     return NULL;
 }
 
 /* Keeps track of one connection to the gateway. */
 
-/* GABA:
+/* ;;GABA:
    (class
      (name gateway)
      (vars
