@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.162 2002/03/20 13:06:33 nisse Exp $ */
+ * $Id: io.c,v 1.166 2002/06/02 15:45:45 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -260,7 +260,7 @@ lsh_oop_stop_callback(oop_source *source UNUSED,
   
   if (!nfiles)
     /* An arbitrary non-NULL value stops oop_sys_run. */
-    return &nfiles;
+    return OOP_HALT;
   else
     return OOP_CONTINUE;
 }
@@ -316,6 +316,12 @@ io_final(void)
   oop_sys_delete(global_oop_sys);
   global_oop_sys = NULL;
   source = NULL;
+
+  /* It's a good idea to reset std* to blocking mode. */
+
+  io_set_blocking(STDIN_FILENO);
+  io_set_blocking(STDOUT_FILENO);
+  io_set_blocking(STDERR_FILENO);
 }
 
 void
@@ -664,12 +670,14 @@ static struct io_callback io_write_callback =
 
 struct listen_value *
 make_listen_value(struct lsh_fd *fd,
-		  struct address_info *peer)
+		  struct address_info *peer,
+		  struct address_info *local)
 {
   NEW(listen_value, self);
 
   self->fd = fd;
   self->peer = peer;
+  self->local = local;
 
   return self;
 }
@@ -708,11 +716,13 @@ do_listen_callback(struct io_callback *s,
       werror("io.c: accept failed, %z", STRERROR(errno));
       return;
     }
+
   trace("io.c: accept on fd %i\n", conn);
   COMMAND_RETURN(self->c,
 		 make_listen_value(make_lsh_fd(conn, "accepted socket", self->e),
 				   sockaddr2info(addr_len,
-						 (struct sockaddr *) &peer)));
+						 (struct sockaddr *) &peer), 
+				   fd2info(fd,0)));
 }
 
 struct io_callback *
@@ -1066,6 +1076,35 @@ sockaddr2info(size_t addr_len UNUSED,
     }
 }
 
+struct address_info *
+fd2info(struct lsh_fd *fd, int side)
+{
+#if WITH_IPV6
+  struct sockaddr_storage sock;
+#else
+  struct sockaddr sock;
+#endif
+  
+  socklen_t s_len = sizeof(sock);
+
+  int get;
+
+  if( !side ) /* Local */
+    get = getsockname( fd->fd, (struct sockaddr *)  &sock, &s_len );
+  else
+    get = getpeername( fd->fd, (struct sockaddr *)  &sock, &s_len );
+
+        
+  if (get < 0)  
+    {               
+      werror("io.c: getXXXXname failed, %z", STRERROR(errno));
+      return NULL;
+    }
+
+  return sockaddr2info(s_len,
+		       (struct sockaddr *) &sock);
+}
+
 #if HAVE_GETADDRINFO
 static struct addrinfo *
 choose_address(struct addrinfo *list,
