@@ -1,6 +1,6 @@
 /* keyexchange.c
  *
- * $Id: keyexchange.c,v 1.79 2001/12/12 18:42:59 nisse Exp $ */
+ * $Id: keyexchange.c,v 1.85 2002/03/14 15:35:08 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -30,8 +30,8 @@
 #include "command.h"
 #include "connection.h"
 #include "debug.h"
-#include "disconnect.h"
 #include "format.h"
+#include "io.h"
 #include "parse.h"
 #include "publickey_crypto.h"
 #include "ssh.h"
@@ -47,14 +47,17 @@
 
 #include "keyexchange.c.x"
 
+/* Define this to get very frequent re-exchanges */
+#ifndef STRESS_KEYEXCHANGE
+# define STRESS_KEYEXCHANGE 0
+#endif
+
 /* GABA:
    (class
      (name kexinit_handler)
      (super packet_handler)
      (vars
-       (init object make_kexinit)
-
-       ; Extra rgument for the KEYEXCHANGE_INIT call.
+       ; Extra argument for the KEYEXCHANGE_INIT call.
        (extra object lsh_object)
        
        ; Maps names to algorithms. It's dangerous to lookup random atoms
@@ -147,33 +150,45 @@ format_kex(struct kexinit *kex)
 }
   
 void
-initiate_keyexchange(struct ssh_connection *connection)
+send_kexinit(struct ssh_connection *connection)
 {
   struct lsh_string *s;
   int mode = connection->flags & CONNECTION_MODE;
-  
-  struct kexinit *kex = connection->kexinits[mode];
 
+  struct kexinit *kex
+    = connection->kexinits[mode]
+    = MAKE_KEXINIT(connection->kexinit);
+  
   assert(kex->first_kex_packet_follows == !!kex->first_kex_packet);
-  assert(connection->kex_state == KEX_STATE_INIT);
-	 
+  assert(connection->read_kex_state == KEX_STATE_INIT);
+
+  /* First, disable any key reexchange timer */
+  if (connection->key_expire)
+    {
+      KILL_RESOURCE(connection->key_expire);
+      connection->key_expire = NULL;
+    }
+  
   s = format_kex(kex);
 
   /* Save value for later signing */
 #if 0
-  debug("initiate_keyexchange: Storing literal_kexinits[%i]\n", mode);
+  debug("send_kexinit: Storing literal_kexinits[%i]\n", mode);
 #endif
   
   connection->literal_kexinits[mode] = s; 
+  connection_send_kex_start(connection);  
 
-  C_WRITE(connection, lsh_string_dup(s));
+  C_WRITE_NOW(connection, lsh_string_dup(s));
 
+  /* NOTE: This feature isn't fully implemented, as we won't tell
+   * the selected key exchange method if the guess was "right". */
   if (kex->first_kex_packet_follows)
     {
       s = kex->first_kex_packet;
       kex->first_kex_packet = NULL;
 
-      C_WRITE(connection, s);
+      C_WRITE_NOW(connection, s);
     }
 }
 
@@ -229,7 +244,7 @@ do_handle_kexinit(struct packet_handler *c,
 
   verbose("Received KEXINIT message. Key exchange initated.\n");
   
-  if (connection->kex_state != KEX_STATE_INIT)
+  if (connection->read_kex_state != KEX_STATE_INIT)
     {
       PROTOCOL_ERROR(connection->e, "Unexpected KEXINIT message.");
       return;
@@ -258,18 +273,7 @@ do_handle_kexinit(struct packet_handler *c,
   
   /* Have we sent a kexinit message already? */
   if (!connection->kexinits[mode])
-    {
-      struct lsh_string *packet;
-      struct kexinit *sent = MAKE_KEXINIT(closure->init);
-      connection->kexinits[mode] = sent;
-      packet = format_kex(sent);
-#if 0
-      debug("do_handle_kexinit: Storing literal_kexinits[%i]\n", mode);
-#endif
-      connection->literal_kexinits[mode] = lsh_string_dup(packet); 
-      
-      C_WRITE(connection, packet);
-    }
+    send_kexinit(connection);
 
   /* Select key exchange algorithms */
 
@@ -281,14 +285,14 @@ do_handle_kexinit(struct packet_handler *c,
       kex_algorithm_atom
 	= LIST(connection->kexinits[CONNECTION_CLIENT]->kex_algorithms)[0];
 
-      connection->kex_state = KEX_STATE_IN_PROGRESS;
+      connection->read_kex_state = KEX_STATE_IN_PROGRESS;
     }
   else
     {
       if (msg->first_kex_packet_follows)
 	{
 	  /* Wrong guess */
-	  connection->kex_state = KEX_STATE_IGNORE;
+	  connection->read_kex_state = KEX_STATE_IGNORE;
 	}
 
       /* FIXME: Ignores that some keyexchange algorithms require
@@ -360,15 +364,13 @@ do_handle_kexinit(struct packet_handler *c,
 }
 
 struct packet_handler *
-make_kexinit_handler(struct make_kexinit *init,
-		     struct lsh_object *extra,
+make_kexinit_handler(struct lsh_object *extra,
 		     struct alist *algorithms)
 {
   NEW(kexinit_handler, self);
 
   self->super.handler = do_handle_kexinit;
 
-  self->init = init;
   self->extra = extra;
   self->algorithms = algorithms;
   
@@ -397,15 +399,15 @@ kex_make_key(struct hash_instance *secret,
   if (!key_length)
     return key;
   
-  hash = HASH_COPY(secret);
-  digest = alloca(hash->hash_size);
+  hash = hash_copy(secret);
+  digest = alloca(HASH_SIZE(hash));
 
-  HASH_UPDATE(hash, 1, tags + type); 
-  HASH_UPDATE(hash, session_id->length, session_id->data);
-  HASH_DIGEST(hash, digest);
+  hash_update(hash, 1, tags + type); 
+  hash_update(hash, session_id->length, session_id->data);
+  hash_digest(hash, digest);
 
   /* Is one digest large anough? */
-  if (key_length <= hash->hash_size)
+  if (key_length <= HASH_SIZE(hash))
     memcpy(key->data, digest, key_length);
 
   else
@@ -414,7 +416,7 @@ kex_make_key(struct hash_instance *secret,
       UINT8 *dst = key->data;
       
       KILL(hash);
-      hash = HASH_COPY(secret);
+      hash = hash_copy(secret);
       
       for (;;)
 	{
@@ -428,27 +430,27 @@ kex_make_key(struct hash_instance *secret,
 	  struct hash_instance *tmp;
 	  
 	  /* Append digest to the key data. */
-	  memcpy(dst, digest, hash->hash_size);
-	  dst += hash->hash_size;
-	  left -= hash->hash_size;
+	  memcpy(dst, digest, HASH_SIZE(hash));
+	  dst += HASH_SIZE(hash);
+	  left -= HASH_SIZE(hash);
 
 	  /* And to the hash state */
-	  HASH_UPDATE(hash, hash->hash_size, digest);
+	  hash_update(hash, HASH_SIZE(hash), digest);
 
-	  if (left <= hash->hash_size)
+	  if (left <= HASH_SIZE(hash))
 	    break;
 	  
 	  /* Get a new digest, without disturbing the hash object (as
 	   * we'll need it again). We use another temporary hash for
 	   * extracting the digest. */
 
-	  tmp = HASH_COPY(hash);
-	  HASH_DIGEST(tmp, digest);
+	  tmp = hash_copy(hash);
+	  hash_digest(tmp, digest);
 	  KILL(tmp);
 	}
 
       /* Get the final digest, and use some of it for the key. */
-      HASH_DIGEST(hash, digest);
+      hash_digest(hash, digest);
       memcpy(dst, digest, left);
     }
   KILL(hash);
@@ -567,6 +569,42 @@ kex_make_inflate(struct object_list *algorithms,
   return algorithm ? MAKE_INFLATE(algorithm) : NULL;
 }
 
+/* GABA:
+   (class
+     (name reexchange_timeout)
+     (super lsh_callback)
+     (vars
+       (connection object ssh_connection)))
+*/
+
+static void
+do_reexchange_timeout(struct lsh_callback *s)
+{
+  CAST(reexchange_timeout, self, s);
+  assert(!self->connection->send_kex_only);
+
+  verbose("Session key expired. Initiating key re-exchange.\n");
+  send_kexinit(self->connection);
+}
+
+static void
+set_reexchange_timeout(struct ssh_connection *connection,
+		       unsigned seconds)
+{
+  NEW(reexchange_timeout, timeout);
+
+  verbose("Setting session key lifetime to %i seconds\n",
+	  seconds);
+  timeout->super.f = do_reexchange_timeout;
+  timeout->connection = connection;
+
+  assert(!connection->key_expire);
+  connection->key_expire = io_callout(&timeout->super,
+				      seconds);
+  
+  remember_resource(connection->resources, connection->key_expire); 
+}
+
 /* GABA:
    (class
      (name newkeys_handler)
@@ -577,6 +615,19 @@ kex_make_inflate(struct object_list *algorithms,
        (compression object compress_instance)))
 */
 
+/* Maximum lifetime for the session keys. Use longer timeout on
+ * the server side. */
+
+#if STRESS_KEYEXCHANGE
+# define SESSION_KEY_LIFETIME_CLIENT 4
+# define SESSION_KEY_LIFETIME_SERVER 14
+#else
+/* 40 minutes */
+# define SESSION_KEY_LIFETIME_CLIENT 2400
+/* 90 minutes */
+# define SESSION_KEY_LIFETIME_SERVER 5400
+#endif
+
 static void
 do_handle_newkeys(struct packet_handler *c,
 		  struct ssh_connection *connection,
@@ -598,7 +649,7 @@ do_handle_newkeys(struct packet_handler *c,
       connection->rec_mac = closure->mac;
       connection->rec_compress = closure->compression;
 
-      connection->kex_state = KEX_STATE_INIT;
+      connection->read_kex_state = KEX_STATE_INIT;
 
       connection->kexinits[CONNECTION_CLIENT]
 	= connection->kexinits[CONNECTION_SERVER] = NULL;
@@ -609,8 +660,20 @@ do_handle_newkeys(struct packet_handler *c,
       connection->literal_kexinits[CONNECTION_CLIENT]
 	= connection->literal_kexinits[CONNECTION_SERVER] = NULL;
 
-      connection->dispatch[SSH_MSG_NEWKEYS] = NULL;
+      /* Normally, packet entries in the dispatch table must never be
+       * NULL, but SSH_MSG_NEWKEYS is handled specially by
+       * connection.c:connection_handle_packet. So we could use NULL
+       * here, but for uniformity we don't do that. */
+      
+      connection->dispatch[SSH_MSG_NEWKEYS] = &connection_fail_handler;
 
+      /* Set maximum lifetime for the session keys. Use longer timeout on
+       * the server side. */
+      
+      set_reexchange_timeout
+	(connection,
+	 ((connection->flags & CONNECTION_MODE) == CONNECTION_SERVER)
+	 ? SESSION_KEY_LIFETIME_SERVER : SESSION_KEY_LIFETIME_CLIENT);
       KILL(closure);
     }
   else
@@ -791,27 +854,28 @@ install_keys(struct object_list *algorithms,
  * This mechanism changed in the transport-05 draft. Before this, the
  * exchange hash was not included at this point. */
 static struct hash_instance *
-kex_build_secret(struct hash_algorithm *H,
+kex_build_secret(const struct hash_algorithm *H,
 		 struct lsh_string *exchange_hash,
 		 struct lsh_string *K)
 {
   /* We include a length field for the key, but not for the exchange
    * hash. */
   
-  struct hash_instance *hash = MAKE_HASH(H);
+  struct hash_instance *hash = make_hash(H);
   struct lsh_string *s = ssh_format("%S%lS", K, exchange_hash);
 
-  HASH_UPDATE(hash, s->length, s->data);
+  hash_update(hash, s->length, s->data);
   lsh_string_free(s);
   
   return hash;
 }
 
+
 /* NOTE: Consumes both the exchange_hash and K */
 void
 keyexchange_finish(struct ssh_connection *connection,
 		   struct object_list *algorithms,
-		   struct hash_algorithm *H,
+		   const struct hash_algorithm *H,
 		   struct lsh_string *exchange_hash,
 		   struct lsh_string *K)
 {
@@ -820,8 +884,9 @@ keyexchange_finish(struct ssh_connection *connection,
   /* Send a newkeys message, and install a handler for receiving the
    * newkeys message. */
 
-  C_WRITE(connection, ssh_format("%c", SSH_MSG_NEWKEYS));
-
+  assert(connection->send_kex_only);
+  C_WRITE_NOW(connection, ssh_format("%c", SSH_MSG_NEWKEYS));
+  
   /* A hash instance initialized with the key, to be used for key
    * generation */
   hash = kex_build_secret(H, exchange_hash, K);
@@ -845,18 +910,9 @@ keyexchange_finish(struct ssh_connection *connection,
 
   KILL(hash);
 
-  connection->kex_state = KEX_STATE_NEWKEYS;
-
-#if DATAFELLOWS_WORKAROUNDS
-  if (! (connection->peer_flags & PEER_SEND_NO_DEBUG))
-#endif
-    send_verbose(connection->write, "Key exchange successful!", 0);
-
-  if (connection->established)
-    {
-      struct command_continuation *c = connection->established;
-      connection->established = NULL;
+  /* If any messages were queued during the key exchange, send them
+   * now. */
+  connection_send_kex_end(connection);
   
-      COMMAND_RETURN(c, connection);
-    }
+  connection->read_kex_state = KEX_STATE_NEWKEYS;
 }
