@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: keyexchange.h,v 1.51 2001/07/06 20:13:10 nisse Exp $ */
+ * $Id: keyexchange.h,v 1.55 2002/03/14 10:14:28 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -47,19 +47,6 @@
 
 #define KEX_PARAMETERS 6
 
-/* A KEX_INIT msg can be accepted. This is true, most of the time. */
-#define KEX_STATE_INIT 0
-
-/* Ignore next packet */
-#define KEX_STATE_IGNORE 1
-
-/* Key exchange is in progress. Neither KEX_INIT or NEWKEYS messages
- * can be received */
-#define KEX_STATE_IN_PROGRESS 2
-
-/* Key exchange is finished. A NEWKEYS message should be received, and
- * nothing else. */
-#define KEX_STATE_NEWKEYS 3
 
 #define GABA_DECLARE
 #include "keyexchange.h.x"
@@ -145,11 +132,13 @@ make_simple_kexinit(struct randomness *r,
 extern struct command_2 kexinit_filter;
 #define KEXINIT_FILTER (&kexinit_filter.super.super)
 
-void initiate_keyexchange(struct ssh_connection *connection);
+/* Sends the keyexchange message, which must already be stored in
+ * connection->kexinits[connection->flags & CONNECTION_MODE]
+ */
+void send_kexinit(struct ssh_connection *connection);
 
 struct packet_handler *
-make_kexinit_handler(struct make_kexinit *init,
-		     struct lsh_object *extra,
+make_kexinit_handler(struct lsh_object *extra,
 		     struct alist *algorithms);
 
 struct packet_handler *
@@ -160,7 +149,7 @@ make_newkeys_handler(struct crypto_instance *crypto,
 void
 keyexchange_finish(struct ssh_connection *connection,
 		   struct object_list *algorithms,
-		   struct hash_algorithm *H,
+		   const struct hash_algorithm *H,
 		   struct lsh_string *exchange_hash,
 		   struct lsh_string *K);
 
