@@ -1,6 +1,6 @@
 /* proxy.h
  *
- * $Id: proxy.h,v 1.3 2000/02/06 22:20:21 nisse Exp $ */
+ * $Id: proxy.h,v 1.5 2001/07/04 18:32:04 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -27,12 +27,13 @@
 #include "alist.h"
 #include "command.h"
 
-extern struct collect_info_1 chain_connections;
-#define CHAIN_CONNECTIONS (&chain_connections.super.super.super)
+extern struct command_3 chain_connections;
+#define CHAIN_CONNECTIONS (&chain_connections.super.super)
 
 struct command *
-make_proxy_connection_service(struct alist *server_requests,
-			      struct alist *client_requests);
+make_proxy_connection_service(struct object_list *server_hooks,
+			      struct object_list *client_hooks);
+
 
 struct command *
 make_proxy_offer_service(struct alist *services);
