@@ -1,6 +1,6 @@
 /* proxy_userauth.c
  *
- * $Id: proxy_userauth.c,v 1.7 2001/02/25 22:38:19 nisse Exp $ */
+ * $Id: proxy_userauth.c,v 1.12 2001/04/05 16:21:15 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -22,13 +22,16 @@
  */
 
 #include "proxy_userauth.h"
+
+#include "client_userauth.h"
+#include "format.h"
 #include "proxy.h"
 #include "server_userauth.h"
-#include "client_userauth.h"
-#include "xalloc.h"
 #include "ssh.h"
-#include "lsh.h"
 #include "werror.h"
+#include "xalloc.h"
+
+#include <assert.h>
 
 #define GABA_DEFINE
 #include "proxy_userauth.h.x"
@@ -71,6 +74,7 @@ do_forward_password_userauth(struct proxy_userauth *ignored UNUSED,
       (password = parse_string_copy(args)) &&
       parse_eod(args))
     {
+      /* FIXME: Should we really pass the last argument free=1 ? */
       C_WRITE(connection->chain, format_userauth_password(username, service, password, 1));
     }
 }
@@ -105,14 +109,11 @@ do_forward_success(struct packet_handler *c,
     {
       struct lsh_string *name = self->name;
       self->name = NULL;
-      C_WRITE(connection->chain, packet);
+      C_WRITE(connection->chain, lsh_string_dup(packet));
       COMMAND_RETURN(self->c, make_proxy_user(name));
     }
   else
-    {
-      PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_USERAUTH_SUCCESS message.");
-      lsh_string_free(packet);
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_USERAUTH_SUCCESS message.");
 }
 
 
@@ -161,17 +162,13 @@ do_forward_failure(struct packet_handler *c,
 	= STATIC_EXCEPTION(EXC_USERAUTH,
 			   "Server authentication error.");
 
-      lsh_string_free(packet);
       verbose("Authentication failure");
       
       EXCEPTION_RAISE(closure->e, &userauth_failed);
 
     }
   else
-    {
-      PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_USERAUTH_FAILURE message.");
-      lsh_string_free(packet);
-    }
+    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_USERAUTH_FAILURE message.");
 }
 
 static struct packet_handler *
@@ -231,10 +228,14 @@ do_handle_userauth(struct packet_handler *c,
 	{
 	  connection->chain->dispatch[SSH_MSG_USERAUTH_FAILURE] = 
 	    make_forward_failure(closure->e);
-	  
+
+#if 0
 	  connection->chain->dispatch[SSH_MSG_USERAUTH_SUCCESS] =
 	    make_forward_success(user, make_delay_continuation(service, closure->c));
-
+#endif
+	  connection->chain->dispatch[SSH_MSG_USERAUTH_SUCCESS] =
+	    make_forward_success(user, closure->c);
+	  
 	  PROXY_AUTH(auth, connection, user, requested_service, &buffer);
 	}
       else
@@ -248,8 +249,6 @@ do_handle_userauth(struct packet_handler *c,
     }
   else
     PROTOCOL_ERROR(connection->e, "Invalid USERAUTH message.");
-
-  lsh_string_free(packet);
 }
 
 static struct packet_handler *
@@ -284,16 +283,19 @@ do_proxy_userauth_continuation(struct command_continuation *c,
 			       struct lsh_object *x)
 {
   CAST(proxy_userauth_continuation, self, c);
-  CAST(delayed_apply, action, x);
+  CAST(proxy_user, user, x);
   int i;
 
+  assert(user);
+  /* self->connection->user = user; */
+  
   connection_unlock(self->connection);
 
   /* Ignore any further userauth messages. */
   for (i = SSH_FIRST_USERAUTH_GENERIC; i < SSH_FIRST_CONNECTION_GENERIC; i++) 
     self->connection->dispatch[i] = &connection_ignore_handler;
   
-  FORCE_APPLY(action, self->super.up, self->super.e);
+  COMMAND_RETURN(self->super.up, self->connection);
 
   /* FIXME: Possibly call connection_handle_pending. */
 }
@@ -327,10 +329,8 @@ do_userauth_proxy(struct command *s,
   connection->dispatch[SSH_MSG_USERAUTH_REQUEST] =
     make_proxy_userauth_handler(self->methods,
 				self->services, 
-				make_once_continuation
-				(NULL, 
-				 make_proxy_userauth_continuation
-				 (connection, c, e)),
+				make_proxy_userauth_continuation
+				(connection, c, e),
 				make_exc_userauth_handler(connection, 
 							  self->advertised_methods,
 							  AUTH_ATTEMPTS, e,
