@@ -1,6 +1,6 @@
 /* proxy_userauth.h
  *
- * $Id: proxy_userauth.h,v 1.1 1999/11/29 20:53:54 nisse Exp $ */
+ * $Id: proxy_userauth.h,v 1.2 2000/01/12 22:40:15 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -40,9 +40,11 @@
        (name string)))
 */
 
-extern struct userauth proxy_password_auth;
+struct proxy_userauth;
 
-struct command *make_userauth_proxy(struct int_list *allowed_methods,
+extern struct proxy_userauth proxy_password_auth;
+
+struct command *make_proxy_userauth(struct int_list *allowed_methods,
 				    struct alist *methods,
 				    struct alist *services);
 
