@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: read_line.c,v 1.29 1999/10/24 20:29:24 nisse Exp $ */
+ * $Id: read_line.c,v 1.30 2000/08/04 00:04:50 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -46,7 +46,7 @@
        (e object exception_handler)
        
        ; Line buffer       
-       (pos simple UINT32)
+       (pos . UINT32)
        (buffer array UINT8 MAX_LINE)))
 */
 
