@@ -1,6 +1,6 @@
 /* server_session.h
  *
- * $Id: server_session.h,v 1.15 2001/08/26 14:39:35 nisse Exp $
+ * $Id: server_session.h,v 1.18 2001/09/27 15:49:17 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -39,15 +39,14 @@ make_server_session(UINT32 initial_window,
 struct channel_open *
 make_open_session(struct alist *session_requests);
 
-struct channel_request *
-make_shell_handler(struct io_backend *backend);
+extern struct channel_request
+shell_request_handler;
+
+extern struct channel_request 
+exec_request_handler;
 
 struct channel_request *
-make_exec_handler(struct io_backend *backend);
-
-struct channel_request *
-make_subsystem_handler(struct io_backend *backend,
-		       const char **subsystems);
+make_subsystem_handler(const char **subsystems);
 
 struct lsh_string *
 format_exit_signal(struct ssh_channel *channel,
@@ -58,4 +57,7 @@ format_exit(struct ssh_channel *channel, int value);
 extern struct channel_request 
 pty_request_handler;
 
+extern struct channel_request
+window_change_request_handler;
+
 #endif /* LSH_SERVER_SESSION_H_INCLUDED */
