@@ -2,7 +2,7 @@
  *
  * An implementation of Ron Rivest's S-expressions, used in spki.
  *
- * $Id: sexp.h,v 1.35 2001/09/27 12:14:04 nisse Exp $ */
+ * $Id: sexp.h,v 1.36 2002/01/14 01:14:42 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -77,8 +77,11 @@
 #define SEXP_TRANSPORT 1
 #define SEXP_ADVANCED 2
 
+/* Like advanced, but use hex instead of base64 */
+#define SEXP_ADVANCED_HEX 3
+
 /* Like advanced, but allow international characters in quoted strings. */
-#define SEXP_INTERNATIONAL 3
+#define SEXP_INTERNATIONAL 4
 
 struct lsh_string *sexp_format(struct sexp *e, int style, unsigned indent);
 
