@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: commands.c,v 1.5 2002/03/18 21:07:57 nisse Exp $
+ * @(#) $Id: commands.c,v 1.10 2002/05/26 10:58:07 _cvs_pont Exp $
  *
  * commands.c
  */
@@ -51,8 +51,13 @@ command commands[] = {
   { "about", com_about, "Display information about lsftp", 
     "Displays some information about lsftp.", NOARG, 1, 1 },  
 
-  { "help", com_help, "Display this text", 
-    "help also displays more specific information for certain commands.", COMMAND, 1, 1 },
+  { "help", com_help, "List available commands or help for command", 
+    "help also displays more specific information for certain commands,\n"
+    "as you have likely already discovered.", COMMAND, 1, 1 },
+
+  { "longhelp", com_longhelp, "Give more help", 
+    "If given a command, this is equal to help (help actually uses longhelp for this).\n"
+    "used without arguments, longhelp displays short help texts for all commands.", COMMAND, 1, 2 },
 
   { "?", com_help, "Synonym for `help'", 
     "You can type help COMMAND, but you've already discovered that\n", 
@@ -189,8 +194,50 @@ com_connected(void)
   return -1;
 }
 
+
+int       
+com_help(const char *arg, const char *command UNUSED)
+{ 
+  const char *s;
+  
+  s = lsftp_s_skip(arg," \n\t\r"); /* Skip any initial whitespace */
+  
+  if( !s || ! (*s) ) /* End of string? */
+    {
+      int row = 0;
+      char* cmdname;
+      int i=0;
+      
+      /* No command given, show the short description */
+
+      while( ( cmdname = commands[i++].name) )
+	{
+          printf( "%-10s",      /* Print each command */
+		  cmdname
+		  );
+	  row++;
+
+	  if( 8 == row )
+	    {
+	      printf("\n");
+	      row = 0;
+	    }
+	}
+
+      if( row ) /* Anything printed on this line? */
+	printf("\n");
+
+      printf("\nFor more help, try longhelp or help <COMMAND>\n");
+      return 0;
+    }
+  else /* Command given, show long help */
+    return com_longhelp( arg, command );
+
+}
+
+
 int
-com_help(const char *arg, const char *command UNUSED) 
+com_longhelp(const char *arg, const char *command UNUSED) 
 {
   const char *s;
   char tmp[PATH_MAX];
@@ -203,12 +250,11 @@ com_help(const char *arg, const char *command UNUSED)
     {
       /* No command given, show the short description */
 
-      while( ( cmdname = commands[i].name) )
-	if( commands[i++].visible )   /* Allow for "hidden" commands */
-	  printf("%-10s -- %s\n",
-		 cmdname, 
-		 commands[i-1].doc
-		 ); 
+      while( ( cmdname = commands[i++].name) )
+	printf("%-10s -- %s\n",
+	       cmdname, 
+	       commands[i-1].doc
+	       ); 
       /* Print command name (10 columns) -- documentation */
         
       return 0;
@@ -232,7 +278,7 @@ com_help(const char *arg, const char *command UNUSED)
 	  {
 	    known=1;
 	    printf(
-		   "%-10s -- %s\n%s\n", 
+		   "%-10s -- %s\n\n%s\n", 
 		   commands[i-1].name, 
 		   commands[i-1].doc,
 		   commands[i-1].longdoc
@@ -307,7 +353,9 @@ com_escape(const char *arg, const char *command UNUSED)
 int
 com_quit(const char *arg UNUSED, const char *command UNUSED) 
 {
-  com_close( "", "INTERNAL CLOSE COMMAND" );
+  if( lsftp_connected() )   /* If connected */
+    com_close( "", "INTERNAL CLOSE COMMAND" ); /* Close connection */
+
   printf( "Bye\n" );
 
   mainloop=0;
@@ -430,9 +478,14 @@ com_cd(const char *arg, const char *command UNUSED)
     {
       const char **mglob;
       const char *ptr = 0;
-      
-      mglob = lsftp_dc_r_startglob( tmp, 0, 1 );
-      
+      char* tmp2 = lsftp_qualify_path( tmp );
+
+      if( !tmp2 )
+	return -1;
+
+      mglob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+      free( tmp2 );
+
       if( mglob ) /* Glob returned allright? */
 	ptr = *mglob;
       
@@ -627,7 +680,14 @@ com_rm(const char *arg, const char *command)
 	    const char **orgglob;
 	    const char *ptr;
 	    
-	    mglob = lsftp_dc_r_startglob( tmp, 0, 1 );
+	    char* tmp2 = lsftp_qualify_path( tmp );
+
+	    if( !tmp2 )
+	      return -1;
+
+
+	    mglob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+	    free( tmp2 );
 	    orgglob = mglob;
 
 	    if( mglob ) /* Glob returned allright? */		
@@ -872,8 +932,6 @@ get_file_or_folder(const char *arg, const char *command,
 
   ret = lsftp_dc_l_isdir( tmp );
 
-  printf( "Arg: %s, tmp: %s\n", arg, tmp );
-
   if( !ret )       /* Exists, but not a dir? */
     {
       printf( 
@@ -1050,7 +1108,14 @@ com_get(const char *arg, const char *command)
 	const char **orgglob;
 	const char *ptr;
 
-	glob = lsftp_dc_r_startglob( tmp, 0, 1 );
+	char* tmp2 = lsftp_qualify_path( tmp );
+
+	if( !tmp2 )
+	  return -1;
+
+	glob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+	free( tmp2 );
+
 	orgglob = glob;
 	
 	while( (ptr = *glob++) ) 
@@ -1510,7 +1575,14 @@ com_mv(const char *arg, const char *command)
 	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
 	 )
     {
-      glob = lsftp_dc_r_startglob( tmp, 0, 1 );
+      char* tmp2 = lsftp_qualify_path( tmp );
+
+      if( !tmp2 )
+	return -1;
+
+      glob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+      free( tmp2 );
+
       orgglob = glob;
       
       while( (ptr = *glob++) ) 
@@ -1535,7 +1607,14 @@ com_mv(const char *arg, const char *command)
 	 ( arg = lsftp_s_strtok( arg," \n\t\r", tmp, PATH_MAX ) ) 
 	 )
     {
-      glob = lsftp_dc_r_startglob( tmp, 0, 1 );
+      char* tmp2 = lsftp_qualify_path( tmp );
+
+      if( !tmp2 )
+	return -1;
+
+      glob = lsftp_dc_r_startglob( tmp2, 0, 1 );
+      free( tmp2 );
+
       orgglob = glob;
       
       while( (ptr = *glob++) ) 
@@ -1949,7 +2028,7 @@ handle_command(const unsigned char *s)
     return com_escape( s+1, entered_cmd );
 
   if( s && s[0] == '?' && isalnum( s[1] ) ) /* Commands is ?x... ? */
-    return com_help( s+1, entered_cmd );
+    return com_longhelp( s+1, entered_cmd );
 
   /* Get the first word from the line (the command) */
 
