@@ -1,6 +1,6 @@
 /* sftp-server.c
  *
- * $Id: sftp-server.c,v 1.16 2002/02/11 22:55:31 _cvs_pont Exp $
+ * $Id: sftp-server.c,v 1.19 2002/04/02 16:34:53 _cvs_pont Exp $
  *
  * The server side of the sftp subsystem. */
 
@@ -50,15 +50,17 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
    
-#include "buffer.h"
+#include "io.h"
 #include "sftp.h"
 
 #include "filemode.h"
 #include "idcache.h"
 #include "xmalloc.h"
+#include "werror.h"
 
 #include <assert.h>
 #include <stdarg.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -73,9 +75,12 @@
 #include <grp.h>
 #include <time.h>
 
+const char *werror_program_name = "sftp-server";
+
 #define SFTP_VERSION 3
 
-#define FATAL(x) do { fputs("sftp-server: " x "\n", stderr); exit(EXIT_FAILURE); } while (0)
+
+#define MAX_DIRENTS_PER_REQUEST 100
 
 #define WITH_DEBUG 1
 
@@ -148,7 +153,7 @@ struct sftp_ctx
 };
 
 static void
-sftp_init(struct sftp_ctx *ctx, FILE *in, FILE *out)
+sftp_init(struct sftp_ctx *ctx, int in, int out)
 {
   unsigned i;
   
@@ -449,11 +454,11 @@ sftp_send_status(struct sftp_ctx *ctx, UINT32 status)
 #if 0
     case SSH_FX_NO_CONNECTION:
       /* We must never send this */
-      FATAL( "Status SSH_FX_NO_CONNECTION" );
+      fatal( "Status SSH_FX_NO_CONNECTION" );
       break;
     case SSH_FX_CONNECTION_LOST:
       /* We must never send this */
-      FATAL( "Status SSH_FX_CONNECTION_LOST" );
+      fatal( "Status SSH_FX_CONNECTION_LOST" );
       break;
 #endif
     case SSH_FX_OP_UNSUPPORTED:    
@@ -557,36 +562,72 @@ static int
 sftp_process_readdir(struct sftp_ctx *ctx)
 {
   struct sftp_dir *dir;
-  struct dirent* entry; 
-  struct stat st;
-  
+  struct dirent *entry; 
+  char* filenames[MAX_DIRENTS_PER_REQUEST];
+  struct stat st[MAX_DIRENTS_PER_REQUEST];
+  int i;
+  int used_entries = 0;
+
   if (!sftp_get_handle_dir(ctx, &dir))
     return sftp_bad_message(ctx);
 
-  /* readdir doesn't modify errno on EOF, so we need to clear it
-   * first. */
-  errno = 0;
+  while (used_entries < MAX_DIRENTS_PER_REQUEST )
+    {
+      /* readdir doesn't modify errno on EOF, so we need to clear it
+       * first. */
+      errno = 0;
+      
+      entry = readdir(dir->dir);
+      
+      if (!entry) /* Something's wrong */
+	{
+	  /* errno == 0 => No more dirents, errno != 0 => Error */
+	 
+	  /* If we have already successfully read entries, ignore this
+	   * situation for the moment and trust it to repeat itself on
+	   * subsequent readdirs */
+
+	  if (!used_entries)   /* First time? */
+	    return (errno ? sftp_send_errno(ctx, errno)
+		    : sftp_send_status(ctx, SSH_FX_EOF)); 
+	  else
+	    break; /* We have data to send, so send it */
+	}
+      else /* Successfully read entry */
+	{
+	  if (sftp_lstat_in_dir(dir->name, entry->d_name, &st[used_entries]) )
+	    /* FIXME: Would it be better to just skip this entry, or send an
+	     * SSH_FXP_NAME with no file attributes? */
+	    {
+	      for(i=0; i<used_entries; i++)
+		free(filenames[i]);
+	      return sftp_send_errno(ctx, errno);
+	    }
+
+	  filenames[used_entries] = strdup(entry->d_name);
+
+	  if (!filenames[used_entries]) /* strdup failed? */
+	    {
+	      for(i=0; i<used_entries; i++)
+		free(filenames[i]);
+	      return sftp_send_errno(ctx, errno);
+	    }
+
+	  used_entries++; /* Everything OK, skip to next entry */
+	}
+    }
+
+  if (!used_entries) /* This should never happen */
+    return sftp_send_status(ctx, SSH_FX_EOF);
+  
+  sftp_put_uint32(ctx->o, used_entries);
+  
+  for (i=0; i<used_entries; i++)
+    {
+      sftp_put_filename(ctx, &st[i], filenames[i]);
+      free(filenames[i]);
+    }
   
-  entry=readdir(dir->dir);
-
-  if (!entry)
-    return (errno ? sftp_send_errno(ctx, errno)
-	    : sftp_send_status(ctx, SSH_FX_EOF)); 
-
-  if (sftp_lstat_in_dir(dir->name, entry->d_name, &st) )
-    /* FIXME: Would it be better to just skip this entry, or send an
-     * SSH_FXP_NAME with no file attributes? */
-    return sftp_send_errno(ctx, errno);
-
-  /* FIXME: we don't have to, but maybe we should be nice and pass
-   * several at once? It might improve performance quite a lot (or it
-   * might not) */
-
-  /* Use count == 1 for now. */
-  sftp_put_uint32(ctx->o, 1);
-
-  sftp_put_filename(ctx, &st, entry->d_name);
-
   sftp_set_msg(ctx->o, SSH_FXP_NAME );
   return 1;
 }
@@ -1262,17 +1303,17 @@ sftp_process(sftp_process_func **dispatch,
   
   /* All packets start with a msg byte and a 32-bit id. */
   if (!sftp_get_uint8(ctx->i, &msg))
-    FATAL("Invalid packet.");
+    fatal("Invalid packet.");
 
   if (!sftp_get_uint32(ctx->i, &id))
-    FATAL("Invalid packet.");
+    fatal("Invalid packet.");
 
   DEBUG (("sftp_process: msg: %d, id: %d\n", msg, id));
 
   /* Every reply starts with the id, so copy it through */
   sftp_set_id(ctx->o, id);
     
-  /* Calls FATAL on protocol errors. */
+  /* Calls fatal on protocol errors. */
   ok = dispatch[msg](ctx);
     
   /* Every handler should result in at least one message */
@@ -1356,7 +1397,7 @@ main(int argc, char **argv)
   if (!parse_options(argc, argv))
     return EXIT_FAILURE;
   
-  sftp_init(&ctx, stdin, stdout);
+  sftp_init(&ctx, STDIN_FILENO, STDOUT_FILENO);
 
   if (!sftp_handshake(&ctx))
     return EXIT_FAILURE;
