@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: sftp_bind.c,v 1.5 2002/02/25 12:14:00 _cvs_pont Exp $
+ * @(#) $Id: sftp_bind.c,v 1.10 2002/05/25 16:22:10 _cvs_pont Exp $
  *
  * sftp_bind.c - bindings for sftp.c
  *
@@ -29,8 +29,8 @@
 
 #include "sftp_bind.h"
 
-static FILE* to_transport;
-static FILE* from_transport;
+#include "io.h"
+#include "werror.h"
 
 static int transport_pid = 0;
 
@@ -113,10 +113,10 @@ int lsftp_open_connection(char** argv, int argc)
     }
 
   if (pipe(stdin_pipe) < 0)
-    FATAL("Creating stdin_pipe failed.");
+    fatal("Creating stdin_pipe failed.");
   
   if (pipe(stdout_pipe) < 0)
-    FATAL("Creating stdout_pipe failed.");
+    fatal("Creating stdout_pipe failed.");
 
 #ifdef USING_CYGWIN
   setmode( stdin_pipe[0], O_BINARY );
@@ -129,27 +129,14 @@ int lsftp_open_connection(char** argv, int argc)
   switch( ( pid = fork() ) )
     {
     case -1:
-      FATAL("fork failed.");
+      fatal("fork failed.");
     default: /* Parent */
       {
-	FILE *i;
-	FILE *o;
 	void* err;
 
 	close(stdin_pipe[0]); /* Close those ends of the pipe we won't use */
 	close(stdout_pipe[1]);
 	
-	i = fdopen(stdout_pipe[0], "r");
-	if (!i)
-	  FATAL("fdopen stdout_pipe failed.");
-	
-	o = fdopen(stdin_pipe[1], "w");
-	if (!o)
-	  FATAL("fdopen stdin_pipe failed.");
-
-	from_transport = i;
-	to_transport = o;
-	
 	fd_from_transport = stdout_pipe[0];
 	fd_to_transport = stdin_pipe[1];
 
@@ -164,6 +151,9 @@ int lsftp_open_connection(char** argv, int argc)
       }
     case 0: /* Child */
       {
+	/* FIXME: Do as much as possible of this setup before fork(),
+	 * for better error reporting. */
+	
 	int k = 0;
 	int j = 0;
 	int i;
@@ -228,7 +218,7 @@ int lsftp_open_connection(char** argv, int argc)
 								    NULL. */
 	
 	if ( !new_argv )
-	  FATAL("Malloc failed ");
+	  fatal("Malloc failed ");
 
 
 	cur_string = before_string;
@@ -274,9 +264,9 @@ int lsftp_open_connection(char** argv, int argc)
       
 	
 	if (dup2(stdin_pipe[0], STDIN_FILENO) < 0)
-	  FATAL("dup2 for stdin failed.");
+	  fatal("dup2 for stdin failed.");
 	if (dup2(stdout_pipe[1], STDOUT_FILENO) < 0)
-	  FATAL("dup2 for stdout failed.");
+	  fatal("dup2 for stdout failed.");
 	
 	close(stdin_pipe[0]);
 	close(stdin_pipe[1]);
@@ -340,7 +330,7 @@ int lsftp_open_connection(char** argv, int argc)
 		 ".\n"
 		 );
 	
-	FATAL( "Could not start a suitable secsh client." );
+	fatal( "Could not start a suitable secsh client." );
       }
     }
 
@@ -368,12 +358,12 @@ int lsftp_close_connection()
 
   signal( SIGCHLD, SIG_DFL ); /* Restore original signal handler before quitting */
 
-  i = fclose( to_transport ); /* Close outgoing */
+  i = close( fd_to_transport ); /* Close outgoing */
 
   /* FIXME: Read what we can here */
   /* FIXME: Should we report any errors here? */
 
-  i = i || fclose( from_transport );
+  i = i || close( fd_from_transport );
 
   if( -1 != kill( transport_pid, SIGTERM ) ) /* Signal sent OK? */
     {
@@ -398,8 +388,8 @@ int lsftp_close_connection()
   alarm( 0 ); /* Cancel any alarm */
 
   transport_pid = 0; /* Make invalid */
-  to_transport = 0;
-  from_transport = 0;
+  fd_to_transport = 0;
+  fd_from_transport = 0;
 
   /* FIXME: Free memory used by sftp_input in and sftp_output out? */
 
@@ -420,8 +410,8 @@ int lsftp_handshake()
 {
   int failed;
 
-  in = sftp_make_input( from_transport ); 
-  out = sftp_make_output( to_transport );
+  in = sftp_make_input( fd_from_transport ); 
+  out = sftp_make_output( fd_to_transport );
 
   failed = sftp_handshake( in, out );
 
@@ -436,7 +426,7 @@ int lsftp_handshake()
       while( readok )
 	{
 	  char x;
-	  readok = fread( &x, 1, 1, from_transport ); /* Try to read one byte */
+	  readok = read( fd_from_transport, &x, 1 ); /* Try to read one byte */
 	  fprintf( stderr, "%c", x);
 	}
 
@@ -1269,8 +1259,25 @@ lsftp_do_cd(const char *dir)
   return 0;
 }
 
+
+int 
+lsftp_path_is_absolute( const char* path )
+{
+  /* Return 0 if the path given is not absolute */
+
+  /* FIXME: Assumes the path is absolute iff it begins with / */
+
+  if( path && 
+      ( path[0] == '/' )
+      )
+    return 1;
+
+  return 0;
+}
+
+
 const char *
-lsftp_unqualify_path(const char *path )
+lsftp_unqualify_path( const char *path )
 {
   /*
    * Returns the part that was passed to lsftp_qualify_path 
@@ -1309,7 +1316,7 @@ lsftp_qualify_path(const char *path)
    * side (no operation on absoulte paths, adds pwd/ to relative paths
    */
 
-  if( '/' == path[0] ||
+  if( lsftp_path_is_absolute( path ) ||
       !curpath ||
       !curpath[0]
       )
@@ -1459,6 +1466,10 @@ lsftp_do_ls(const char* dir, const char* command, int longlist, int all)
     fnameg = strdup( dir );        /* Only local glob */
   
   glob = lsftp_dc_r_startglob( dglob, 0, 1 ); /* Not sloppy, leave incorrect ones */
+
+  if( !glob ) /* Failed? */
+    return -1; /* Bail out */
+
   orgglob = glob;
 
   if( !strlen( fnameg ) ) /* Empty filename glob? */
