@@ -1,6 +1,6 @@
 /* tty.h
  *
- * $Id: tty.c,v 1.12 2001/04/11 06:10:28 nisse Exp $ */
+ * $Id: tty.c,v 1.14 2001/09/30 18:12:26 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -73,8 +73,8 @@ tty_setwinsize(int fd, const struct terminal_dimensions *dims)
 {
   struct winsize ws;
   
-  ws.ws_row = dims->char_width;
-  ws.ws_col = dims->char_height;
+  ws.ws_col = dims->char_width;
+  ws.ws_row = dims->char_height;
   ws.ws_xpixel = dims->pixel_width;
   ws.ws_ypixel = dims->pixel_height;
   
