@@ -1,4 +1,4 @@
-/* userauth.h
+/* userauth.c
  *
  */
 
@@ -31,17 +31,6 @@
 #include "userauth.h.x"
 #undef GABA_DEFINE
 
-struct lsh_string *format_userauth_failure(struct int_list *methods,
-					   int partial)
-{
-  return ssh_format("%c%A%c", SSH_MSG_USERAUTH_FAILURE, methods, partial);
-}
-
-struct lsh_string *format_userauth_success(void)
-{
-  return ssh_format("%c", SSH_MSG_USERAUTH_SUCCESS);
-}
-
 struct exception *
 make_userauth_special_exception(struct lsh_string *reply, const char *msg)
 {
