@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: werror.c,v 1.63 2001/09/16 19:20:24 nisse Exp $ */
+ * $Id: werror.c,v 1.65 2001/12/12 18:18:49 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -562,6 +562,16 @@ werror(const char *format, ...)
     }
 }
 
+void
+werror_progress(const char *string)
+{
+  if (verbose_flag || !quiet_flag)
+    {
+      werror_write(strlen(string), string);
+      werror_flush();
+    }
+}
+
 void
 trace(const char *format, ...) 
 {
